# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test Attribute."""

import unittest
import uuid

from neural_insights.components.graph.attribute import Attribute
from neural_insights.utils.json_serializer import JsonSerializer


def _get_random_string() -> str:
    """Create random string to be used."""
    return uuid.uuid4().hex


class TestAttribute(unittest.TestCase):
    """Test Attribute class."""

    def test_setting_parameters(self) -> None:
        """Test if parameters are correctly set."""
        name = _get_random_string()
        attribute_type = _get_random_string()
        value = _get_random_string()

        attribute = Attribute(name=name, attribute_type=attribute_type, value=value)

        self.assertEqual(name, attribute.name)
        self.assertEqual(attribute_type, attribute.attribute_type)
        self.assertEqual(value, attribute.value)

    def test_serialization(self) -> None:
        """Test if Attribute is serialized as expected."""
        name = _get_random_string()
        attribute_type = _get_random_string()
        value = _get_random_string()

        attribute = Attribute(name=name, attribute_type=attribute_type, value=value)

        expected_serialized_object = {
            "name": name,
            "attribute_type": attribute_type,
            "value": value,
        }
        serialized = JsonSerializer.serialize_item(attribute)
        self.assertEqual(expected_serialized_object, serialized)


if __name__ == "__main__":
    unittest.main()
